class _BotCommands:
    def __init__(self):
        self.StartCommand = 'start'
        self.MirrorCommand = 'mirr'
        self.UnzipMirrorCommand = 'unzipmirr'
        self.TarMirrorCommand = 'tarmirror'
        self.CancelMirrorCommand = 'cancel'
        self.CancelAllCommand = 'cancelall'
        self.ListCommand = 'list'
        self.StatusCommand = 'status'
        self.AuthorizeCommand = 'auth'
        self.UnAuthorizeCommand = 'unauth'
        self.AuthListCommand = 'authlist'
        self.PingCommand = 'ping'
        self.RestartCommand = 'restart'
        self.StatsCommand = 'stats'
        self.HelpCommand = 'help'
        self.LogCommand = 'log'
        self.CloneCommand = 'clone'
        self.WatchCommand = 'watch'
        self.SpeedCommand = 'speedtest'
        self.UsageCommand = 'usage'
        self.TarWatchCommand = 'tarwatch'
        self.DeleteCommand = 'delete'
        self.ConfigCommand = 'config'


BotCommands = _BotCommands()
